@@ -100,15 +100,19 @@ TimerThread *gTimerThread;
 ThreadPool gRecvThreadPool;
 /*! Mini server thread pool. */
 ThreadPool gMiniServerThreadPool;
-static constexpr std::array<std::pair<ThreadPool*, const char*>, 3> o_threadpools{{
-    {&gSendThreadPool, "Send thread pool"},
-    {&gRecvThreadPool, "Receive thread pool"},
-    {&gMiniServerThreadPool, "Mini server thread pool"},
-}};
+using tpooldesc = std::pair<ThreadPool*, const char*>;
+static const std::array<std::pair<ThreadPool*, const char*>, 3> o_threadpools{
+    tpooldesc{&gSendThreadPool, "Send thread pool"},
+    tpooldesc{&gRecvThreadPool, "Receive thread pool"},
+    tpooldesc{&gMiniServerThreadPool, "Mini server thread pool"},
+};
 
 /*! Flag to indicate the state of web server */
 WebServerState bWebServerState = WEB_SERVER_DISABLED;
 
+WebCallback_HostValidate g_hostvalidatecallback;
+void *g_hostvalidatecookie;
+
 /* Interfaces we are using */
 std::vector<NetIF::Interface> g_netifs;
 /* Small optimisation: if the interfaces parameter to UpnpInit2() was
@@ -637,18 +641,18 @@ exit_function:
     return retVal;
 }
 
-int UpnpInit(const char *hostIP, unsigned short DestPort)
+EXPORT_SPEC int UpnpInit(const char *hostIP, unsigned short DestPort)
 {
     return upnpInitCommon(hostIP, nullptr, DestPort);
 }
 
-int UpnpInit2(const char *IfName, unsigned short DestPort)
+EXPORT_SPEC int UpnpInit2(const char *IfName, unsigned short DestPort)
 {
     return UpnpInitWithOptions(IfName, DestPort,
                                UPNP_FLAG_IPV6, UPNP_OPTION_END);
 }
 
-int UpnpInit2(const std::vector<std::string>& ifnames, unsigned short port)
+EXPORT_SPEC int UpnpInit2(const std::vector<std::string>& ifnames, unsigned short port)
 {
     // A bit wasteful, but really simpler. Just build an interfaces
     // list string and continue with this.
@@ -747,7 +751,7 @@ static UPNP_INLINE void PrintThreadPoolStats(
 }
 #endif /* DEBUG */
 
-int UpnpFinish()
+EXPORT_SPEC int UpnpFinish()
 {
 #ifdef INCLUDE_DEVICE_APIS
     UpnpDevice_Handle device_handle;
@@ -795,7 +799,41 @@ int UpnpFinish()
     return UPNP_E_SUCCESS;
 }
 
-unsigned short UpnpGetServerPort()
+EXPORT_SPEC int UpnpSetWebRequestHostValidateCallback(
+    WebCallback_HostValidate callback, void *cookie)
+{
+    g_hostvalidatecallback = callback;
+    g_hostvalidatecookie = cookie;
+    return UPNP_E_SUCCESS;
+}
+
+EXPORT_SPEC std::string UpnpGetUrlHostPortForClient(const struct sockaddr_storage* clsock)
+{
+    NetIF::IPAddr claddr(reinterpret_cast<const struct sockaddr*>(clsock));
+    NetIF::IPAddr hostaddr;
+    const NetIF::Interface *itf =
+        NetIF::Interfaces::interfaceForAddress(claddr, g_netifs, hostaddr);
+    if (nullptr == itf) {
+        return std::string();
+    }
+    int port = 0;
+    std::string prefix;
+    switch (hostaddr.family()) {
+    case NetIF::IPAddr::Family::IPV4:
+        port = UpnpGetServerPort();
+        break;
+    case NetIF::IPAddr::Family::IPV6:
+        prefix = "[";
+        port = UpnpGetServerPort6();
+            break;
+    default:
+        return std::string();
+    }
+
+    return prefix + hostaddr.straddr() + (prefix.empty() ? "" : "]") + ":" + lltodecstr(port);
+}
+
+EXPORT_SPEC unsigned short UpnpGetServerPort()
 {
     if (UpnpSdkInit != 1)
         return 0U;
@@ -803,17 +841,27 @@ unsigned short UpnpGetServerPort()
     return LOCAL_PORT_V4;
 }
 
-#ifdef UPNP_ENABLE_IPV6
-unsigned short UpnpGetServerPort6()
+EXPORT_SPEC unsigned short UpnpGetServerPort6()
 {
+#ifdef UPNP_ENABLE_IPV6
     if (UpnpSdkInit != 1)
         return 0U;
 
     return LOCAL_PORT_V6;
-}
+#else
+    return 0;
 #endif
+}
+EXPORT_SPEC unsigned short UpnpGetServerUlaGuaPort6()
+{
+#ifdef UPNP_ENABLE_IPV6
+        return 0;
+#else
+        return 0;
+#endif
+}
 
-const char *UpnpGetServerIpAddress()
+EXPORT_SPEC const char *UpnpGetServerIpAddress()
 {
     if (UpnpSdkInit != 1)
         return "";
@@ -824,8 +872,9 @@ const char *UpnpGetServerIpAddress()
     return addr.c_str();
 }
 
-const char *UpnpGetServerIp6Address()
+EXPORT_SPEC const char *UpnpGetServerIp6Address()
 {
+#ifdef UPNP_ENABLE_IPV6
     if (UpnpSdkInit != 1 || !using_ipv6()) {
         return "";
     }
@@ -834,9 +883,12 @@ const char *UpnpGetServerIp6Address()
         addr = apiFirstIPV6Str();
     }
     return addr.c_str();
+#else
+    return nullptr;
+#endif
 }
 
-const char *UpnpGetServerUlaGuaIp6Address()
+EXPORT_SPEC const char *UpnpGetServerUlaGuaIp6Address()
 {
     return "";
 }
@@ -987,7 +1039,7 @@ exit_function:
     return retVal;
 }
 
-int UpnpRegisterRootDevice(
+EXPORT_SPEC int UpnpRegisterRootDevice(
     const char *DescUrl, Upnp_FunPtr Fun, const void *Cookie,
     UpnpDevice_Handle *Hnd)
 {
@@ -995,7 +1047,7 @@ int UpnpRegisterRootDevice(
         UPNPREG_URL_DESC, DescUrl, Fun, Cookie, Hnd, nullptr);
 }
 
-int UpnpRegisterRootDevice2(
+EXPORT_SPEC int UpnpRegisterRootDevice2(
     Upnp_DescType descriptionType, const char *description_const,
     size_t, int /*ignored*/, Upnp_FunPtr Fun, const void *Cookie,
     UpnpDevice_Handle *Hnd)
@@ -1004,7 +1056,7 @@ int UpnpRegisterRootDevice2(
         descriptionType, description_const, Fun, Cookie, Hnd, nullptr);
 }
 
-int UpnpRegisterRootDevice4(
+EXPORT_SPEC int UpnpRegisterRootDevice4(
     const char *DescUrl, Upnp_FunPtr Fun, const void *Cookie,
     UpnpDevice_Handle *Hnd, int /*AddressFamily*/, const char *LowerDescUrl)
 {
@@ -1012,7 +1064,7 @@ int UpnpRegisterRootDevice4(
         UPNPREG_URL_DESC, DescUrl, Fun, Cookie, Hnd, LowerDescUrl);
 }
 
-int UpnpDeviceSetProduct(
+EXPORT_SPEC int UpnpDeviceSetProduct(
     UpnpDevice_Handle Hnd, const char *product, const char *version)
 {
     struct Handle_Info *HInfo = nullptr;
@@ -1030,12 +1082,12 @@ int UpnpDeviceSetProduct(
     return UPNP_E_SUCCESS;
 }
 
-int UpnpUnRegisterRootDevice(UpnpDevice_Handle Hnd)
+EXPORT_SPEC int UpnpUnRegisterRootDevice(UpnpDevice_Handle Hnd)
 {
     return UpnpUnRegisterRootDeviceLowPower(Hnd, -1, -1, -1);
 }
 
-int UpnpUnRegisterRootDeviceLowPower(UpnpDevice_Handle Hnd, int PowerState,
+EXPORT_SPEC int UpnpUnRegisterRootDeviceLowPower(UpnpDevice_Handle Hnd, int PowerState,
                                      int SleepPeriod, int RegistrationState)
 {
     UpnpPrintf(UPNP_DEBUG,API, __FILE__, __LINE__, "UpnpUnRegisterRootDevice\n");
@@ -1835,8 +1887,7 @@ int UpnpDownloadUrlItem(const char *url, char **outBuf, char *contentType)
 
     if (url == nullptr || outBuf == nullptr)
         return UPNP_E_INVALID_PARAM;
-    ret_code = http_Download(url, HTTP_DEFAULT_TIMEOUT, outBuf, &dummy,
-                             contentType);
+    ret_code = http_Download(url, HTTP_DEFAULT_TIMEOUT, outBuf, &dummy, contentType);
     if (ret_code > 0)
         /* error reply was received */
         ret_code = UPNP_E_INVALID_URL;
