@@ -65,6 +65,17 @@
 #endif /* _WIN32 */
 
 
+#ifndef IN6_IS_ADDR_GLOBAL
+#define IN6_IS_ADDR_GLOBAL(a) \
+    ((((__const uint32_t *) (a))[0] & htonl(0x70000000)) == htonl (0x20000000))
+#endif /* IS ADDR GLOBAL */
+
+#ifndef IN6_IS_ADDR_ULA
+#define IN6_IS_ADDR_ULA(a) \
+        ((((__const uint32_t *) (a))[0] & htonl(0xfe000000)) == htonl (0xfc000000))
+#endif /* IS ADDR ULA */
+
+
 namespace NetIF {
 
 static FILE *logfp;
@@ -118,25 +129,24 @@ IPAddr& IPAddr::operator=(const IPAddr& o)
 IPAddr::IPAddr(const char *caddr)
     : IPAddr()
 {
-    if (std::strchr(caddr, ':') != nullptr) {
-        if (inet_pton(
-                AF_INET6, caddr,
-                &reinterpret_cast<struct sockaddr_in6*>(m->saddr)->sin6_addr)
-            == 1) {
+    if (nullptr != std::strchr(caddr, ':')) {
+        if (inet_pton(AF_INET6, caddr,
+                      &reinterpret_cast<struct sockaddr_in6*>(m->saddr)->sin6_addr) == 1) {
             m->saddr->sa_family = AF_INET6;
             m->ok = true;
         }
     } else {
         if (inet_pton(AF_INET, caddr,
-                      &reinterpret_cast<struct sockaddr_in*>(m->saddr)->sin_addr)
-            == 1) {
+                      &reinterpret_cast<struct sockaddr_in*>(m->saddr)->sin_addr) == 1) {
             m->saddr->sa_family = AF_INET;
             m->ok = true;
         }
     }
 }
 
-IPAddr::IPAddr(const struct sockaddr *sa)
+static const uint8_t ipv4mappedprefix[12] = {0,0,0,0,0,0,0,0,0,0,0xff,0xff};
+
+IPAddr::IPAddr(const struct sockaddr *sa, bool unmapv4)
     : IPAddr()
 {
     switch (sa->sa_family) {
@@ -145,9 +155,24 @@ IPAddr::IPAddr(const struct sockaddr *sa)
         m->ok = true;
         break;
     case AF_INET6:
+    {
+        if (unmapv4) {
+            const uint8_t *bytes =
+                reinterpret_cast<const struct sockaddr_in6 *>(sa)->sin6_addr.s6_addr;
+            if (!memcmp(bytes, ipv4mappedprefix, 12)) {
+                struct sockaddr_in *a = reinterpret_cast<struct sockaddr_in*>(m->saddr);
+                memset(a, 0, sizeof(*a));
+                a->sin_family = AF_INET;
+                memcpy(&a->sin_addr.s_addr, bytes+12, 4);
+                m->ok = true;
+                break;
+            }
+        }
+        // unmapv4==false or not a v4 mapped address, copy v6 address
         memcpy(m->saddr, sa, sizeof(struct sockaddr_in6));
         m->ok = true;
-        break;
+    }
+    break;
     default:
         break;
     }
@@ -210,18 +235,48 @@ IPAddr::Scope IPAddr::scopetype() const
         return Scope::Invalid;
     if (family() != Family::IPV6)
         return Scope::Invalid;
+
+    // Unrouted link-local address. If there are several interfaces on
+    // the host, an additional piece of information (scope id) is
+    // necessary to determine what interface they belong too.
+    // e.g. fe80::1 could exist on both eth0 and eth1, and needs
+    // scopeid 0/1 for complete determination
     if (IN6_IS_ADDR_LINKLOCAL(
             &(reinterpret_cast<struct sockaddr_in6*>(m->saddr))->sin6_addr)) {
         return Scope::LINK;
     }
+
+    // Site-local addresses are deprecated. Prefix fec0, then locally
+    // chosen network and interface ids. Routable within the
+    // site. They also need a site/scope ID, always 1 if there is only
+    // one site defined.
     if (IN6_IS_ADDR_SITELOCAL(
             &(reinterpret_cast<struct sockaddr_in6*>(m->saddr))->sin6_addr)) {
         return Scope::SITE;
     }
+
+    // We process unique local addresses and global ones in the same way.
     return Scope::GLOBAL;
 }
 
+bool IPAddr::setScopeIdx(const IPAddr& other)
+{
+    if (family() != Family::IPV6 || other.family() != Family::IPV6 ||
+        scopetype() != Scope::LINK || other.scopetype() != Scope::LINK) {
+        return false;
+    }
+    struct sockaddr_in6 *msa6 = reinterpret_cast<struct sockaddr_in6*>(m->saddr);
+    struct sockaddr_in6 *osa6 = reinterpret_cast<struct sockaddr_in6*>(other.m->saddr);
+    msa6->sin6_scope_id = osa6->sin6_scope_id;
+    return true;
+}
+
 std::string IPAddr::straddr() const
+{
+    return straddr(false, false);
+}
+
+std::string IPAddr::straddr(bool setscope, bool forurl) const
 {
     if (!ok())
         return std::string();
@@ -231,14 +286,22 @@ std::string IPAddr::straddr() const
     switch(m->saddr->sa_family) {
     case AF_INET:
         inet_ntop(m->saddr->sa_family,
-                  &reinterpret_cast<struct sockaddr_in*>(m->saddr)->sin_addr,
-                  buf, 200);
+                  &reinterpret_cast<struct sockaddr_in*>(m->saddr)->sin_addr, buf, 200);
     break;
     case AF_INET6:
-        inet_ntop(m->saddr->sa_family,
-                  &reinterpret_cast<struct sockaddr_in6*>(m->saddr)->sin6_addr,
-                  buf, 200);
-        break;
+    {
+        struct sockaddr_in6 *sa6 = reinterpret_cast<struct sockaddr_in6*>(m->saddr);
+        inet_ntop(m->saddr->sa_family, &sa6->sin6_addr, buf, 200);
+        if (!setscope || scopetype() != Scope::LINK) {
+            return buf;
+        }
+        std::string s{buf};
+        char scopebuf[30];
+        snprintf(scopebuf, sizeof(scopebuf), "%u", sa6->sin6_scope_id);
+        s += std::string(forurl ? "%25" : "%") + scopebuf;
+        return s;
+    }
+    break;
     }
     return buf;
 }
